#!/bin/bash

################################################################################
#       Copyright (C) 2023, International Business Machines Corp. (IBM)        #
#                          All rights reserved.                                #
#                                                                              #
#       Automated Distribution of SP Server Certificate to Clients             #
#                                                                              #
# This BASH script is the automation utility, responsible for distributing the #
# Spectrum Protect server certificate to all BA and TDP clients. Eliminating   #
# the need of log-in on each client machine connected to server and manually   #
# adding the new certificate.                                                  #
#                                                                              #
# This utility consists of a BASH script file cert_distribute.sh and           #
# the corresponding configuration file cert_distribute.ini                     # 
#                                                                              #
# Prerequisites:                                                               #
# 1. The ADMIN CLIENT (dsmadmc) running on Linux OS has been configured to be  #
#    able to connect to the Spectrum Protect server.                           #
# 2. The client scheduler has been configured for all BA and TDP clients.      #
#                                                                              #
# Procedure:                                                                   #
#                                                                              #
# 1. Generate a new certificate on Spectrum Protect server for distributing    #
#    to all clients using CREATE CERTIFICATE command.                          #
#                                                                              #
# 2. Copy the new generated certificate file from server to admin client.      #
#    You can skip this step if the server and the admin client are installed   #
#    on the same box.                                                          #
#                                                                              #
# 3. Place this script and cert_distribute.ini file in the same directory on   #
#    the admin client box, and modify the settings in cert_distribute.ini file #
#    as appropriate.                                                           #
#                                                                              #
# 4. Execute this script with argument "-action distribute" to START the job   #
#    of certificate distribution. A set of client schedules will be defined    #
#    against all BA and TDP clients.                                           #
#                                                                              #
# 5. Execute this script with argument "-action report" to monitor PROGRESS of #
#    the certificate distribution job. The distribute schedules will be run on #
#                                                                              #
#    This will generate the report with current status of distribution process #
#    of all BA and TDP clients.                                                #
#                                                                              #
#    Schedules defined in the previous step will be executed by the client     #
#    scheduler on a daily basis.                                               #
#                                                                              #
#    Leave those schedules to run for certain days.                            #
#                                                                              #
#    Report will show status as "Completed" for clients where certificate      #
#    is distributed successfully.                                              #
#                                                                              #
#    For clients with status other than "Completed", give them a chance to     #
#    retry by waiting for few more days, as the scheduler will execute those   #
#    schedules again.                                                          #
#                                                                              #
#    NOTE:                                                                     #
#    Some clients might still not be seen with status as "Completed" after     #
#    certain days of retries, either because of client scheduler not running   #
#    or some other issues. Those clients are considered failed. Please go th-  #
#    rough the manual steps to distribute certification for the failed clients.#
#                                                                              #
# 7. When the certificate distribute job is considered done,                   #
#    i.e. for all clients report file shows status as "Completed".             #
#    Execute this script with argument "-action cleanup" to remove all defined #
#    schedules from SP server.                                                 #
#                                                                              #
# POST-EXECUTION STEPS: i.e after successful distribution of the certificate   #
# on all BA and TDP Clients.                                                   #
#                                                                              #
# To set the distributed server certificate to default:                        #
# 1. Execute command :                                                         #
#                                                                              #
#    SET DEFAULTTLSCERT <certificate-label>                                    #
#                                                                              #
#    Here, in above command, "certificate-label" is the label mentioned in the #
#    configurations file cert_distribute.ini used while executing the script.  #
#                                                                              #
# 2. RESTART the Spectrum Protect server to get the new certificate in effect  #
#    for further communication with all BA and TDP clients.                    #
#                                                                              #
# SCRIPT USAGE Examples :                                                      #
# To start a certificate distribution job:                                     #
# ./cert_distribute.sh -id dsmadmc-username                                    #
#                      -password dsmadmc-user-password                         #
#                      -action distribute                                      #
#                                                                              #
# To monitor status of existing certificate distribution job:                  #
# ./cert_distribute.sh -id dsmadmc-username                                    #
#                      -password dsmadmc-user-password                         #
#                      -action report                                          #
#                                                                              #
# To cleanup existing certificate distribution job:                            #
# ./cert_distribute.sh -id dsmadmc-username                                    #
#                      -password dsmadmc-user-password                         #
#                      -action cleanup                                         #
#                                                                              #
################################################################################

action=""
certificateLabel=""
certText=""
configFile='cert_distribute.ini'
currentDir=""
domainPlatformPairs=()
macroDEFINEcmdOutputFile='macroDEFINEcmdOutput.out'
macroDELETEcmdOutputFile='macroDELETEcmdOutput.out'
dsmadmcpath=''
dsmoptfile=''
id=""
isSection=0
LOGFILE="cert_distribute.log"
macroFile="certdistribute.macro"
mandateKeys=("action" "id" "password")
mandatoryCounter=0
newClientSideCertificateFile=''
newcertfile=''
nodeListFile="nodelist.out"
password=""
platformstrwin="WIN"
platformstrlnx="LNX"
platformstraix="AIX"
platformstrmac="MAC"
platformstrunx="UNX"
platformstrtbd="TBD"
reportPrefix='certdistribute.report.'
sectionName=''
sectionNamePattern='\[(.*)\]'
scheddelay=0
schedduration=0
schedulePart00=''
schedulePart01=''
schedulePart02=''
schedulePart03=''
scheduleprefix=''
startdelay=0
supportedPlatforms=("WIN" "UNX" "LNX" "AIX" "MAC")
tempCertificateFileName=""
validActions=("distribute" "report" "cleanup")
varValuePAttern='(.*)=(.*)'

function updateOutputFilesPath(){
    # Making sure all output files are present in same directory.
    curDir=$1
    suffix='/'
    if [[ $curDir == *$suffix ]]; then
        configFile="${curDir}${configFile}"
        macroDEFINEcmdOutputFile="${curDir}${macroDEFINEcmdOutputFile}"
        macroDELETEcmdOutputFile="${curDir}${macroDELETEcmdOutputFile}"
        LOGFILE="${curDir}${LOGFILE}"
        macroFile="${curDir}${macroFile}"
        nodeListFile="${curDir}${nodeListFile}"
    else
        configFile="${curDir}/${configFile}"
        macroDEFINEcmdOutputFile="${curDir}/${macroDEFINEcmdOutputFile}"
        macroDELETEcmdOutputFile="${curDir}/${macroDELETEcmdOutputFile}"
        LOGFILE="${curDir}/${LOGFILE}"
        macroFile="${curDir}/${macroFile}"
        nodeListFile="${curDir}/${nodeListFile}"
    fi
}

function getTimeStamp(){
    timeStamp=$(date +%Y%m%d%H%M%S)
    echo $timeStamp
}

function LOG(){
    DATETIME=$(getTimeStamp)
    LOGTYPE=$1
    if [ "$LOGTYPE" == "INFO" ] || [ "$LOGTYPE" == "WARN" ]; then
        LOGTYPE="$LOGTYPE " # Adding space to align text entries in log file correctly.
    fi
    LOGMESSAGE=$2
    printf "%s : %s %s\n" "$DATETIME" "$LOGTYPE" "$LOGMESSAGE" >> $LOGFILE
}

function updateVariables() {
    upVarRetCode=0
    case "$1" in  
        "dsmadmcpath")  
            dsmadmcpath=$2
            ;;  
        "dsmoptfile")  
            dsmoptfile=$2  
            ;;  
        "scheduleprefix")  
            scheduleprefix=$2  
            ;;  
        "newcertfile")  
            newcertfile=$2  
            ;;  
        "action")  
            action=$2  
            ;;  
        "id")  
            id=$2  
            ;;  
        "pa")  
            password=$2  
            ;;  
        "pass")  
            password=$2  
            ;;  
        "password")  
            password=$2  
            ;;  
        "startdelay")
            startdelay=$2
            ;;
        "scheddelay")
            scheddelay=$2
            ;;
        "schedduration")
            schedduration=$2
            ;;
        *)  
            upVarRetCode=1
            ;;  
    esac
    if [ "$upVarRetCode" -eq "1" ]; then
        echo ""
        echo "ERROR : Unknown parameter '$1'. Please confirm input arguments provided while executing this utility and set in configurations INI file are correct."
        LOG "ERROR" "Unknown parameter '$1'. Please confirm input arguments provided while executing this utility and set in configurations INI file are correct."
        echo ""
        printUsage
        exit 1
    fi
}

function printVariables() {
    echo "dsmadmcpath    = $dsmadmcpath"
    echo "dsmoptfile     = $dsmoptfile"
    echo "newcertfile    = $newcertfile"
}

function validateConfigPaths(){
    # Check if paths in INI file given are correct.
    if [ ! -d $dsmadmcpath ]; then
        echo ""
        echo "ERROR : Invalid PATH set to administrative client (dsmadmc) installation directory in configurations INI file. Please confirm if values set in configurations INI file are correct."
        LOG "ERROR" "ERROR : Invalid PATH set to administrative client (dsmadmc) installation directory in configurations INI file. Please confirm if values set in configurations INI file are correct."
        echo ""
        echo ">> PATHs from INI file are:"
        printVariables
        exit 1
    fi

    if [ ! -f $dsmoptfile ]; then
        echo ""
        echo "ERROR : Invalid PATH set to administrative client (dsmadmc) OPTIONS File in configurations INI file. Please confirm if values set in configurations INI file are correct."
        LOG "ERROR" "ERROR : Invalid PATH set to administrative client (dsmadmc) OPTIONS File in configurations INI file. Please confirm if values set in configurations INI file are correct."
        echo ""
        echo ">> PATHs from INI file are:"
        printVariables
        exit 1
    fi
}

function printUsage(){
    echo "--------------------------------------------------------------------------------"
    echo "Utility to distribute Spectrum Protect Server Certificate to Clients."
    echo ""
    echo "./cert_distribute.sh -id dsmadmc_user -password dsmadmc_user_password -action action_to_perform"
    echo ""
    echo "Mandetory paramenters to execute this script are."
    echo ""
    echo "    id       : Username for logging into DSMADMC console."
    echo ""
    echo "    password : Password for logging into DSMADMC console."
    echo ""
    echo "    action   :"
    echo "               distribute - To dispatch certificates to all nodes."
    echo "               report     - To monitor status of existing certification distribute job."
    echo "               cleanup    - To cleanup existing certification distribute job."
    echo "--------------------------------------------------------------------------------"
}

function isValidAction(){
    if [ "$action" == "distribute" ] || [ "$action" == "report" ] || [ "$action" == "cleanup" ]; then
        echo 0
    else
        echo 1
    fi
}

function checkArguments(){
    LOG "INFO" "checkArguments - Checking input arguments to script."

    if [ ! "$#" == "6" ]; then
        echo ""
        echo "ERROR : Incorrect number of arguments were provided while executing script."
        LOG "ERROR" "Incorrect number of arguments were provided while executing script."
        echo ""
        printUsage
        exit 1
    fi

    variables=()
    values=()

    for ARGUMENT in "$@"
    do
        if [[ $ARGUMENT == -* ]]; then
            variables+=( "$ARGUMENT" )
        else
            values+=( "$ARGUMENT" )
        fi
    done

    for n in {0..2}; 
    do
        KEY=$(echo "${variables[n]}" | cut -f2 -d-)
        if [[ ${mandateKeys[@]} =~ $KEY ]]; then
            VALUE="${values[n]}"
            mandatoryCounter=$((mandatoryCounter+1))
            updateVariables $KEY $VALUE
        fi
    done
    
    if [ "$mandatoryCounter" -lt "${#mandateKeys[*]}" ]; then
        echo ""
        echo "ERROR : One or more of the mandatory parameters were not provided.";
        LOG "ERROR" "Mandatory parameters were not provided while executing script."
        echo ""
        printUsage
        exit 1
    fi
    result=$(isValidAction)
    if [ "$result" == "1" ]; then
        echo ""
        echo "ERROR : Invalid value provided for parameter named action.";
        LOG "ERROR" "Invalid value provided for parameter named action."
        echo ""
        printUsage
        exit 1
    fi
}

function readConfigFile() {
    LOG "INFO" "readConfigFile - Reading file $file."
    while read line; do
    if [[ $line = '#'* ]]; then
        continue
    fi

    if [[ $line == *"["*"]"* ]]; then
        isSection=1
        if  [[ $line =~ $sectionNamePattern ]]; then
            sectionName=${BASH_REMATCH[1]}
        fi
    fi

    if [[ $line == *"="* ]]; then
        if  [[ $line =~ $varValuePAttern ]]; then            
            variable=$(echo "${BASH_REMATCH[1]}" | sed 's/^[ \t]*//;s/[ \t]*$//')
            value=$(echo "${BASH_REMATCH[2]}" | sed 's/^[ \t]*//')
            updateVariables $variable $value
        fi
        continue
    fi
    done < $configFile

    if [[ $line == *"="* ]]; then
        if  [[ $line =~ $varValuePAttern ]]; then
            variable=$(echo "${BASH_REMATCH[1]}" | sed 's/^[ \t]*//;s/[ \t]*$//')
            value=$(echo "${BASH_REMATCH[2]}" | sed 's/^[ \t]*//')
            updateVariables $variable $value
        fi
    fi

}

function getNodesSQLStatement(){
    local sql="select distinct nd.domain_name, case when upper(substr(nd.platform_name,1,3))='WIN' or upper(substr(nd.client_os_name,1,3))='WIN' or upper(substr(nd1.platform_name,1,3))='WIN' or upper(substr(nd1.client_os_name,1,3))='WIN' or substr(fs.FILESPACE_NAME,1,1)='\' then 'WIN' when upper(substr(nd.platform_name,1,3)) in ('UNI','AIX') or upper(substr(nd1.client_os_name,1,3))='AIX' or upper(substr(nd1.platform_name,1,3)) in ('UNI','AIX') or upper(substr(nd1.client_os_name,1,3))='AIX' then 'AIX' when upper(substr(nd.platform_name,1,3))='MAC' or upper(substr(nd.client_os_name,1,3))='MAC' or upper(substr(nd1.platform_name,1,3))='MAC' or upper(substr(nd1.client_os_name,1,3))='MAC' then 'MAC' when upper(substr(nd.platform_name,1,3)) in ('LIN','LNX') or upper(substr(nd.client_os_name,1,3)) in ('LNX','SOL','HPX') or upper(substr(nd1.platform_name,1,3)) in ('LIN','LNX') or upper(substr(nd1.client_os_name,1,3)) in ('LNX','SOL','HPX') or substr(fs.FILESPACE_NAME,1,1)='/' then 'LNX' else 'TBD' end as platform, nd.node_name from nodes nd left join filespaces fs on fs.NODE_NAME=nd.NODE_NAME left join nodes nd1 on nd1.proxy_target like '%' || nd.node_name || '%' where nd.nodetype not in ('NAS', 'OBJECTCLIENT') order by nd.domain_name, nd.node_name"
    echo "$sql"
}

function getClientNodes(){
    LOG "INFO" "getClientNodes - Getting client nodes details."
    nodesSQLStatement=$(getNodesSQLStatement)

    cd $dsmadmcpath
    OUTPUT=$(./dsmadmc -id=$1 -password=$2 \
        -DATAONLY=yes -COMMAdelimited \
        -DISPLaymode=TABle -optfile=$dsmoptfile \
        -outfile=$3 \
        $nodesSQLStatement)
    exitCode=$?
    echo "$exitCode"
}

function readFewLines(){
    platformCode=$1
    START_LINE_NO=$2
    END_LINE_NO=$3
    outputString=''
    i=0
    while read line; do
        i=$(( i + 1 ))
        if [ $i -ge $START_LINE_NO ] && [ $i -le $END_LINE_NO ]; then
            if [ $i -eq $END_LINE_NO ]; then
                outputString+="$line"
            else
                outputString+="$line\n"
            fi
        fi
    done <<< "$certText"
    echo "$outputString"
}

function getRandomString(){
    randomString=$(tr -dc A-Za-z0-9 </dev/urandom | head -c 7 ; echo '')
    echo $randomString
}

function getGSKLinuxCommand(){
    cl=$certificateLabel
    cf=$newClientSideCertificateFile
    linuxCommand="z='/dsmcert.kdb';"
    linuxCommand+="y='/opt/tivoli/tsm/client/';"
    linuxCommand+="a=\$PASSWORDDIR\$z;"
    linuxCommand+="b=\$DSM_DIR\$z;"
    linuxCommand+="c=~'/IBM/StorageProtect/certs'\$z;"
    linuxCommand+="d=\$y'ba/bin64'\$z;"
    linuxCommand+="e=\$y'ba/bin'\$z;"
    linuxCommand+="f=\$y'api/bin64'\$z;"
    linuxCommand+="g=\$y'api/bin'\$z;"
    linuxCommand+="if [ -f \$a ]; then h=\$a;"
    linuxCommand+="elif [ -f \$b ]; then h=\$b;"
    linuxCommand+="elif [ -f \$c ]; then h=\$c;"
    linuxCommand+="elif [ -f \$d ]; then h=\$d;"
    linuxCommand+="elif [ -f \$e ]; then h=\$e;"
    linuxCommand+="elif [ -f \$f ]; then h=\$f;"
    linuxCommand+="else h=\$g;"
    linuxCommand+="fi;"
    linuxCommand+="/usr/local/ibm/gsk8_64/bin/gsk8capicmd_64"
    linuxCommand+=" -cert -add  -label ${cl}  -file ${cf}"
    linuxCommand+=" -db \$h -stashed"
    echo $linuxCommand
}

function getGSKAIXCommand(){
    cl=$certificateLabel
    cf=$newClientSideCertificateFile
    aixCommand="z='dsmcert.kdb';"
    aixCommand+="y='/usr/tivoli/tsm/client/';"
    aixCommand+="a='\${PASSWORDDIR}/'\${z};"
    aixCommand+="b='\${DSM_DIR}/'\${z};"
    aixCommand+="d=\${y}'ba/bin64/'\${z};"
    aixCommand+="e=\${y}'ba/bin/'\${z};"
    aixCommand+="f=\${y}'api/bin64/'\${z};"
    aixCommand+="g=\${y}'api/bin/'\${z};"
    aixCommand+="h='';"
    aixCommand+="if [ -f \$a ]; then h=\$a;"
    aixCommand+="elif [ -f \$b ]; then h=\$b;"
    aixCommand+="elif [ -f \$d ]; then h=\$d;"
    aixCommand+="elif [ -f \$e ]; then h=\$e;"
    aixCommand+="elif [ -f \$f ]; then h=\$f;"
    aixCommand+="elif [ -f \$g ]; then h=\$g;"
    aixCommand+="fi;"
    aixCommand+="/usr/opt/ibm/gsk8_64/bin/gsk8capicmd_64"
    aixCommand+=" -cert -add  -label ${cl}  -file ${cf} -db \$h -stashed"
    echo $aixCommand
}

function getGSKMACCommand(){
    cl=$certificateLabel
    cf=$newClientSideCertificateFile
    macCommand="z=dsmcert.kdb;"
    macCommand+="y=/Library/Application\ Support/tivoli/tsm/client/;"
    macCommand+="a=\${PASSWORDDIR}/\${z};"
    macCommand+="b=\${DSM_DIR}/\${z};"
    macCommand+="e=\${y}ba/bin/\${z};"
    macCommand+="g=\${y}api/bin/\${z};"
    macCommand+="h=\"\";"
    macCommand+="if [ -f \"\$a\" ]; then h=\$a;"
    macCommand+="elif [ -f \"\$b\" ]; then h=\$b;"
    macCommand+="elif [ -f \"\$e\" ]; then h=\$e;"
    macCommand+="elif [ -f \"\$g\" ]; then h=\$g;"
    macCommand+="fi;"
    macCommand+="/Library/ibm/gsk8/bin/gsk8capicmd"
    macCommand+=" -cert -add  -label ${cl} -file ${cf} -db \"\$h\" -stashed"
    echo $macCommand
}

function getGSKWindowsCommand(){
    cl=$certificateLabel
    cf=$newClientSideCertificateFile
    winCommand="powershell \$z='\dsmcert.kdb';\$y=(get-location).path;\$x='C:\Program Files\';\$a=\$(\$env:PASSWORDDIR)+\$z;\$b=\$(\$env:USERPROFILE)+'\IBM\SpectrumProtect\certs'+\$z;\$c=\$(\$env:DSM_DIR)+\$z;\$d=\$x+'Tivoli\TSM\baclient'+\$z;\$e=\$x+'Common Files\Tivoli\TSM\api64'+\$z;if(Test-Path \$a){\$f=\$a}elseif(Test-Path \$b){\$f=\$b}elseif(Test-Path \$c){\$f=\$c}elseif(Test-Path \$d){\$f=\$d}elseif(Test-Path \$e){\$f=\$e};cd 'C:\Program Files\IBM\gsk8\bin';.\gsk8capicmd_64 -cert -add -label $cl -file \$y\\$cf -db \$f -stashed"
    echo $winCommand
}

function getGSKCommand(){
    platform=$1
    gskCommand=""
    if [ "$platform" = "UNX" ]; then
        gskCommand=$(getGSKLinuxCommand)
    elif [ "$platform" = "LNX" ]; then
        gskCommand=$(getGSKLinuxCommand)
    elif [ "$platform" = "AIX" ]; then
        gskCommand=$(getGSKAIXCommand)
    elif [ "$platform" = "MAC" ]; then
        gskCommand=$(getGSKMACCommand)
    elif [ "$platform" = "WIN" ]; then
        gskCommand=$(getGSKWindowsCommand)
    fi
    echo $gskCommand
}

function updateScheduleInMacroFile(){
    domainName=$1
    platformCode=$2
    clientName=$3
    minutes=0
    schedule0=""
    schedule1=""
    schedule2=""
    schedule3=""

    if [ "$platformCode" = "$platformstrwin" ]; then
        minutes=$((1 * scheddelay))
        schedule0="def sch ${domainName} ${scheduleprefix}_0_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj='${schedulePart00}'"
        minutes=$((2 * scheddelay))
        schedule1="def sch ${domainName} ${scheduleprefix}_1_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj='${schedulePart01}'"
        minutes=$((3 * scheddelay))
        schedule2="def sch ${domainName} ${scheduleprefix}_2_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj='${schedulePart02}'"
    else
        minutes=$((1 * scheddelay))
        schedule0="def sch ${domainName} ${scheduleprefix}_0_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj=\"${schedulePart00}\""
        minutes=$((2 * scheddelay))
        schedule1="def sch ${domainName} ${scheduleprefix}_1_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj=\"${schedulePart01}\""
        minutes=$((3 * scheddelay))
        schedule2="def sch ${domainName} ${scheduleprefix}_2_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj=\"${schedulePart02}\""
    fi
    minutes=$((4 * scheddelay))
    if [ "$platformCode" = "$platformstrmac" ]; then
        # Double quotes in commamd of 4th schedule for MAC. Thus inclosing in single quote obj=''.
        schedule3="def sch ${domainName} ${scheduleprefix}_3_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj='${schedulePart03}'"
    else
        schedule3="def sch ${domainName} ${scheduleprefix}_3_${platformCode} act=c startt=now+0${startdelay}:${minutes} DUR=${schedduration} DURU=MINUTES obj=\"${schedulePart03}\""
    fi
    printf "%s\n" "$schedule0" >> $macroFile
    printf "%s\n" "$schedule1" >> $macroFile
    printf "%s\n" "$schedule2" >> $macroFile
    printf "%s\n" "$schedule3" >> $macroFile
}

function deleteScheduleInMacroFile(){
    domainName=$1
    platformCode=$2
    scheduleName="$platformCode"

    for n in {0..3};
    do
        schedule="delete schedule ${domainName} ${scheduleprefix}_${n}_${scheduleName}"
        printf "%s\n" "$schedule" >> $macroFile
    done
}

function updateAssocInMacroFile(){
    domainName=$1
    platformCode=$2
    clientName=$3
    scheduleName="$platformCode"

    for n in {0..3};
    do
        assoc="def assoc $domainName ${scheduleprefix}_${n}_${scheduleName} $clientName"
        printf "%s\n" "$assoc" >> $macroFile
    done
}

function processClientNodes(){

    # Delete Existing MACRO File.
    rm -f $macroFile

    clientInfoFile=$1

    while read line; do
        # Each client node information string in file looks like,
        # domain_name,platform_code,client_name
        # Thus splitting string using comma delimeter to get details.
        arrIN=(${line//,/ })

        domainName="${arrIN[0]}"
        platformCode="${arrIN[1]}"
        clientName="${arrIN[2]}"

        # Unsupported Platform.
        if [[ ! ${supportedPlatforms[@]} =~ $platformCode ]]; then
            # TODO : Check for TDP clients. how it gives platform code.
            LOG "INFO" "Client : $clientName Platform $platformCode is not supported."
            continue
        fi

        # Get certificate text parts with platform specific new line format for echo command.
        tPart01=$(readFewLines $platformCode 1 7)
        tPart02=$(readFewLines $platformCode 8 14)
        tPart03=$(readFewLines $platformCode 15 21)

        if [ "$platformCode" = "$platformstrlnx" ]; then
            schedulePart00="echo -e '${tPart01}' \>> ${newClientSideCertificateFile}"
            schedulePart01="echo -e '${tPart02}' \>> ${newClientSideCertificateFile}"
            schedulePart02="echo -e '${tPart03}' \>> ${newClientSideCertificateFile}"
        elif [ "$platformCode" = "$platformstrwin" ]; then
            # Replace "\n" with "`n" and " " with "` " for Windows ECHO.
            tPart01=$(sed 's/\\n/\`n/g' <<< $tPart01)
            tPart02=$(sed 's/\\n/\`n/g' <<< $tPart02)
            tPart03=$(sed 's/\\n/\`n/g' <<< $tPart03)

            tPart01=$(sed 's/ /\` /g' <<< $tPart01)
            tPart02=$(sed 's/ /\` /g' <<< $tPart02)
            tPart03=$(sed 's/ /\` /g' <<< $tPart03)
            
            schedulePart00="powershell echo \"${tPart01}\" \>> ${newClientSideCertificateFile}"
            schedulePart01="powershell echo \"${tPart02}\" \>> ${newClientSideCertificateFile}"
            schedulePart02="powershell echo \"${tPart03}\" \>> ${newClientSideCertificateFile}"
        else
            schedulePart00="echo '${tPart01}' \>> ${newClientSideCertificateFile}"
            schedulePart01="echo '${tPart02}' \>> ${newClientSideCertificateFile}"
            schedulePart02="echo '${tPart03}' \>> ${newClientSideCertificateFile}"
        fi

        # Update schedule part 4 having GSK command according to platform.
        schedulePart03=$(getGSKCommand $platformCode)

        # For all clients under same domain and platform. Common schedule will be there.
        # check if domain_platform name present in already created schedule name prefix.
        dpPair="$domainName-$platformCode"
        if [[ " ${domainPlatformPairs[@]} " =~ " ${dpPair} " ]]; then
            # Already schedule name created for this pair.
            # Directly associate client with this schedule.
            updateAssocInMacroFile $domainName $platformCode $clientName
        else
            # New entry in array. Create schedule for this and,
            # associate client on this schedule.
            domainPlatformPairs+=("$dpPair")
            updateScheduleInMacroFile $domainName $platformCode $clientName
            updateAssocInMacroFile $domainName $platformCode $clientName
        fi
    done < $clientInfoFile
}

function executeMacroFile(){
    cd $dsmadmcpath
    OUTPUT=$(./dsmadmc -id=$id -password=$password \
        -optfile=$dsmoptfile \
        -outfile=$1 \
        -ITEMCOMMIT macro "$macroFile")
    exitCode=$?
    echo "${exitCode}"
}

function distributeCertificates(){
    LOG "INFO" "distributeCertificates - Starting to distribute certificate to all clients."

    if [ ! -f $newcertfile ]; then
        echo "ERROR : Certificate File not present. Please check pre-requisite steps to execute script again."
        LOG "ERROR" "Certificate File not present. Please check pre-requisite steps to execute script again."
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    # Get certificate text in certText.
    certText=$(cat $newcertfile)

    userName=$id
    password=$password
    exitCode=$(getClientNodes $userName $password $nodeListFile)

    if [ "$exitCode" -ne "0" ] || [ ! -f $nodeListFile ]; then
        echo "ERROR : Error in getting client nodes information. EXIT Code : $exitCode"
        LOG "ERROR" "Getting client nodes information. EXIT Code : $exitCode"
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    # Generate Certificate Label.
    rString=$(getRandomString)
    certificateLabel="spcert_$rString"
    newClientSideCertificateFile="${certificateLabel}.crt"

    processClientNodes $nodeListFile
    if [ ! -f $macroFile ]; then
        echo "Error in defining schedules ${scheduleprefix}*. MACRO file $macroFile not found."
        LOG "ERROR" "Error in defining schedules ${scheduleprefix}*. MACRO file $macroFile not found."
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    exitCode=$(executeMacroFile $macroDEFINEcmdOutputFile)
    
    SPRETCODE=$(grep -E 'ANS8002I' $macroDEFINEcmdOutputFile)
    LOG "INFO" "executeMacroFile  : ${SPRETCODE}"

    if [ "$exitCode" -ne "0" ]; then
        echo ""
        echo "${SPRETCODE}"
        echo ""
        echo "Error in defining schedules ${scheduleprefix}*."
        echo "For more information, take a look at DISTRIBUTE MACRO commands output file $macroDEFINEcmdOutputFile"
        LOG "ERROR" "Error in defining schedules ${scheduleprefix}*. For more information, take a look at DISTRIBUTE MACRO commands output file $macroDEFINEcmdOutputFile"
        echo ""
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    echo ""
    echo "${SPRETCODE}"
    echo ""
    echo ">> Schedules are defined to distribute the certificate to clients."
    echo ">> A return code other than 0 indicates an error. For more information, take a look at the log file."
    echo ">> The log file is located at $LOGFILE"
    echo ">> Client nodes will begin receiving a new certificate after $startdelay hours from the start time of the scheduler window."
    echo ">> To monitor the progress of certificate distribution, it is recommended to run this script in Report mode regularly for a couple of days."
    echo ""
}

function generateReport(){
    LOG "INFO" "generateReport - Starting to generate report."
    timeStamp=$(date +%Y-%m-%d-%H_%M_%S)
    reportFile01="${reportPrefix}_01_${timeStamp}"
    reportFile02="${reportPrefix}_02_${timeStamp}"
    reportFile="${currentDir}/${reportPrefix}${timeStamp}"

    # Check for last schedule status.
    schedpattern="${scheduleprefix}_3_"
    sql="select distinct nd.domain_name, nd.node_name, case when min(ev.status) is not null then min(ev.status) else 'Unknown' end status from nodes nd left join events ev on (ev.domain_name=nd.domain_name and ev.node_name=nd.node_name) where ev.schedule_name is null or upper(ev.schedule_name) like upper('$schedpattern%') group by nd.domain_name,nd.node_name"
    sql2="select status, count(*) node_count from (select distinct nd.domain_name, nd.node_name, case when min(ev.status) is not null then min(ev.status) else 'Unknown' end status from nodes nd left join events ev on (ev.domain_name=nd.domain_name and ev.node_name=nd.node_name) where ev.schedule_name is null or upper(ev.schedule_name) like upper('$schedpattern%') group by nd.domain_name,nd.node_name) group by status"

    cd $dsmadmcpath
    # Calling SQL2 First
    OUTPUT=$(./dsmadmc -id=$id -password=$password \
        -DISPLaymode=TABle \
        -optfile=$dsmoptfile \
        -outfile=$reportFile01 \
        $sql2)
    exitCode=$?
    if [ $exitCode -ne 0 ]; then
        echo "ERROR : Error in getting status of schedule operation on client nodes."
        LOG "ERROR" "Error in getting status of schedule operation on client nodes."
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    # Calling SQL.
    OUTPUT=$(./dsmadmc -id=$id -password=$password \
        -DISPLaymode=TABle \
        -optfile=$dsmoptfile \
        -outfile=$reportFile02 \
        $sql)
    exitCode=$?
    if [ $exitCode -ne 0 ]; then
        echo "ERROR : Error in getting status of schedule operation on client nodes."
        LOG "ERROR" "Error in getting status of schedule operation on client nodes."
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    # Append output from SQL1 and SQL2 in report file.
    cat $reportFile01 >> $reportFile
    cat $reportFile02 >> $reportFile

    LOG "INFO" "The detailed status report is : $reportFile"

    echo ""
    echo "The detailed status report is : $reportFile"
    echo ""
    echo ">> Any status other than 'Completed' indicates a failure."
    echo ">> Client nodes with status ‘Missed’ specifies that the scheduled startup window is already passed."
    echo ">> Client nodes with status ‘Future’ specifies that the beginning of the startup window is in the future."
    echo ">> For ‘Missed’ and ‘Future’ status there is a chance that they will catch up as these schedules will be re-run on a daily basis."
    echo ">> Please regularly monitor the progress of certificate distribution status, by executing this script in Report mode for a couple of days."
    echo ">> Client nodes that are still in failed state, it is recommended to follow the manual steps for adding the certificate."
    echo ">>     1. Copy the new certificate file over to the client box."
    echo ">>     2. Execute below command to add the certificate."
    echo ">>        gsk8capicmd_64 -cert -add -label \"<<New Certificate Label>>\" -file \"<<New Certificate File Path>>\" -db dsmcert.kdb -stashed"
    echo ">>        Tip: The dsmcert.kdb file is located in client installation directory.  On Unix, Linux and Mac systems, if client sessions were ever started from a non-root user, copies of the certificate can be located in \$HOME/IBM/StorageProtect/certs/ or in the directory determined by the PASSWORDDIR client option."
    echo ""
}

function cleanup(){
    LOG "INFO" "cleanup - Starting cleanup."

    # Delete Existing MACRO File.
    rm -f $macroFile

    userName=$id
    password=$password
    exitCode=$(getClientNodes $userName $password $nodeListFile)
    
    if [ "$exitCode" -ne "0" ] || [ ! -f $nodeListFile ]; then
        echo "ERROR : Error in getting client nodes information. EXIT Code : $exitCode"
        LOG "ERROR" "Getting client nodes information. EXIT Code : $exitCode"
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    while read line; do
        # Each client node information string in file looks like,
        # domain_name,platform_code,client_name
        # Thus splitting string using comma delimeter to get details.
        arrIN=(${line//,/ })

        domainName="${arrIN[0]}"
        platformCode="${arrIN[1]}"
        clientName="${arrIN[2]}"

        # Unsupported Platform.
        if [[ ! ${supportedPlatforms[@]} =~ $platformCode ]]; then
            # TODO : Check for TDP clients. how it gives platform code.
            LOG "INFO" "Client : $clientName Platform $platformCode is not supported."
            continue
        fi

        dpPair="$domainName-$platformCode"

        if [[ " ${domainPlatformPairs[@]} " =~ " ${dpPair} " ]]; then
            # Already schedule name created for this pair.
            continue
        else
            # New entry in array. Create schedule for this and,
            # associate client on this schedule.
            domainPlatformPairs+=("$dpPair")
            deleteScheduleInMacroFile $domainName $platformCode
        fi

    done < $nodeListFile

    if [ ! -f $macroFile ]; then
        echo "ERROR in cleaning up schedules ${scheduleprefix}*. Please confirm if distribute action was performed before cleanup."
        LOG "ERROR" "ERROR in cleaning up schedules ${scheduleprefix}*. Please confirm if distribute action was performed before cleanup."
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    exitCode=$(executeMacroFile $macroDELETEcmdOutputFile)

    SPRETCODE=$(grep -E 'ANS8002I' $macroDELETEcmdOutputFile)
    LOG "INFO" "executeMacroFile  : ${SPRETCODE}"

    if [ "$exitCode" -ne "0" ]; then
        echo ""
        echo "${SPRETCODE}"
        echo ""
        echo "ERROR in cleaning up schedules *${scheduleprefix}*."
        echo "For more information, take a look at Cleanup MACRO commands output file $macroDELETEcmdOutputFile"
        LOG "ERROR" "ERROR in cleaning up schedules *${scheduleprefix}*. For more information, take a look at Cleanup MACRO commands output file $macroDELETEcmdOutputFile"
        echo ""
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    echo ""
    echo "${SPRETCODE}"
    echo ""
    echo ">> Cleanup Completed."
    echo ">> A return code other than 0 indicates an error. For more information, take a look at the log file."
    echo ">> The log file is located at $LOGFILE"
    echo ""
}

function execute(){
    currentDir=$(pwd)
    updateOutputFilesPath $currentDir

    LOG "INFO" "Starting Script Execution."

    if [ ! -f $configFile ]; then
        echo "ERROR : Configurations INI File $configFile not present. Please check pre-requisite steps to execute script again."
        LOG "ERROR" "Configurations INI File $configFile not present. Please check pre-requisite steps to execute script again."
        LOG "INFO" "Terminating script execution."
        exit 1
    fi

    checkArguments  "$@"

    readConfigFile

    validateConfigPaths

    if [ "$action" == "distribute" ]; then
        distributeCertificates
    elif [ "$action" == "report" ]; then
        generateReport
    elif [ "$action" == "cleanup" ]; then
        cleanup
    fi

    LOG "INFO" "Script Execution Completed."
}

execute  "$@"


