################################################################################
#                                                                              #
#       Copyright (C) 2023, International Business Machines Corp. (IBM)        #
#                          All rights reserved.                                #
#                                                                              #
#       Automated Distribution of SP Server Certificate to Clients             #
#                                                                              #
# This POWERSHELL script is the automation utility, responsible for distribut- #
# ing the Spectrum Protect server certificate to all BA and TDP clients.       #
# Eliminating the need of log-in on each client machine connected to server    #
# and manually adding the new certificate.                                     #
#                                                                              #
# This utility consists of a POWERSHELL script file cert_distribute.ps1 and    #
# the corresponding configuration file cert_distribute.ini                     #                                              
#                                                                              #
# Prerequisites:                                                               #
# 1. The ADMIN CLIENT (dsmadmc) running on Windows OS has been configured to   #
#    be able to connect to the Spectrum Protect server.                        #
# 2. The client scheduler has been configured for all BA and TDP clients.      #
#                                                                              #
# Procedure:                                                                   #
#                                                                              #
# 1. Generate a new certificate on Spectrum Protect server for distributing    #
#    to all clients using CREATE CERTIFICATE command.                          #
#                                                                              #
# 2. Copy the new generated certificate file from server to admin client.      #
#    You can skip this step if the server and the admin client are installed   #
#    on the same box.                                                          #
#                                                                              #
# 3. Place this script and cert_distribute.ini file in the same directory on   #
#    the admin client box, and modify the settings in cert_distribute.ini file #
#    as appropriate.                                                           #
#                                                                              #
# 4. Execute this script with argument "-action distribute" to START the job   #
#    of certificate distribution. A set of client schedules will be defined    #
#    against all BA and TDP clients.                                           #
#                                                                              #
# 5. Execute this script with argument "-action report" to monitor PROGRESS of #
#    the certificate distribution job. The distribute schedules will be run on #
#                                                                              #
#    This will generate the report with current status of distribution process #
#    of all BA and TDP clients.                                                #
#                                                                              #
#    Schedules defined in the previous step will be executed by the client     #
#    scheduler on a daily basis.                                               #
#                                                                              #
#    Leave those schedules to run for certain days.                            #
#                                                                              #
#    Report will show status as "Completed" for clients where certificate      #
#    is distributed successfully.                                              #
#                                                                              #
#    For clients with status other than "Completed", give them a chance to     #
#    retry by waiting for few more days, as the scheduler will execute those   #
#    schedules again.                                                          #
#                                                                              #
#    NOTE:                                                                     #
#    Some clients might still not be seen with status as "Completed" after     #
#    certain days of retries, either because of client scheduler not running   #
#    or some other issues. Those clients are considered failed. Please go th-  #
#    rough the manual steps to distribute certification for the failed clients.#
#                                                                              #
# 7. When the certificate distribute job is considered done,                   #
#    i.e. for all clients report file shows status as "Completed".             #
#    Execute this script with argument "-action cleanup" to remove all defined #
#    schedules from SP server.                                                 #
#                                                                              #
# POST-EXECUTION STEPS: i.e after successful distribution of the certificate   #
# on all BA and TDP Clients.                                                   #
#                                                                              #
# To set the distributed server certificate to default:                        #
# 1. Execute command :                                                         #
#                                                                              #
#    SET DEFAULTTLSCERT <certificate-label>                                    #
#                                                                              #
#    Here, in above command, "certificate-label" is the label mentioned in the #
#    configurations file cert_distribute.ini used while executing the script.  #
#                                                                              #
# 2. RESTART the Spectrum Protect server to get the new certificate in effect  #
#    for further communication with all BA and TDP clients.                    #
#                                                                              #
# SCRIPT USAGE Examples :                                                      #
# To start a certificate distribution job:                                     #
# powershell .\cert_distribute.ps1 -id dsmadmc-username                        #
#                                  -password dsmadmc-user-password             #
#                                  -action distribute                          #
#                                                                              #
# To monitor status of existing certificate distribution job:                  #
# powershell .\cert_distribute.ps1 -id dsmadmc-username                        #
#                                  -password dsmadmc-user-password             #
#                                  -action report                              #
#                                                                              #
# To cleanup existing certificate distribution job:                            #
# powershell .\cert_distribute.ps1 -id dsmadmc-username                        #
#                                  -password dsmadmc-user-password             #
#                                  -action cleanup                             #
#                                                                              #
################################################################################

param (
    [Parameter(Mandatory=$true)][string]$action,
	[Parameter(Mandatory=$true)][string]$id,
	[Parameter(Mandatory=$true)][string]$pass
)

Function Parse-IniFile ($file) {
  $ini = @{}

  # Create a default section if none exist in the file. Like a java prop file.
  $section = "NO_SECTION"
  $ini[$section] = @{}

  switch -regex -file $file {
    "^\[(.+)\]$" {
      $section = $matches[1].Trim()
      $ini[$section] = @{}
    }
    "^\s*([^#].+?)\s*=\s*(.*)" {
      $name,$value = $matches[1..2]
      # skip comments that start with semicolon:
      if (!($name.StartsWith(";"))) {
        $ini[$section][$name] = $value.Trim()
      }
    }
  }
  $ini
}

function Get-RandomString() {
	param(
		[int]$length=10,
		[char[]]$sourcedata
	)

	for($loop=1; $loop -le $length; $loop++) {
		$TempPassword+=($sourcedata | GET-RANDOM | %{[char]$_})
	}
	return $TempPassword
}

# Read sciprt configuration from cert_distribute.ini
$iniFile = Parse-IniFile .\cert_distribute.ini

$newcertfile = $iniFile.CERT.newcertfile

$dsmadmcpath = $iniFile.TSM.dsmadmcpath
$dsmoptfile = $iniFile.TSM.dsmoptfile
$scheduleprefix = $iniFile.TSM.scheduleprefix

$startdelay = $iniFile.SCHED.startdelay
$scheddelay = $iniFile.SCHED.scheddelay
$schedduration = $iniFile.SCHED.schedduration

#$certfilename = (New-Guid)
$certlabel = "spcert_"+(Get-RandomString -length 7 -sourcedata (48..57 + 65..90 + 97..122))
$certfilename = $certlabel + ".crt"

# Constants
# ----------------

# Save current dir
$workingdir = (get-location).path
# Switch to dsmadmc dir
cd $dsmadmcpath

$homedir = (Get-Item 'ENV:\USERPROFILE').value + "\"
$actiondistribute = "distribute"
$actionreport = "report"
$actioncleanup = "cleanup"

$nodelistfile = $homedir + "spcert_nodelist.out"
$macrofile = $homedir + "spcert_distribute.macro"
$reportprefix = $homedir + "spcert_distribute.report."
$logfile = $homedir + "spcert_distribute.log"
$tempfile = $homedir + "spcert_tempfile.out"

$schedbase = "def sch DOMAIN_NAME SCHED_NAME_PLATFORM_NAME act=c startt=START_TIME DUR=$schedduration DURU=m obj="
$schednamebase = $scheduleprefix + "_"
$schedstartbase = "now+0" + $startdelay + ":"

# This client side script for Windows
$schedfinalwin = "`"powershell `$z='\dsmcert.kdb';`$y=(get-location).path;`$x='C:\Program Files\';`$a=`$(`$env:PASSWORDDIR)+`$z;`$b=`$(`$env:USERPROFILE)+'\IBM\SpectrumProtect\certs'+`$z;`$c=`$(`$env:DSM_DIR)+`$z;`$d=`$x+'Tivoli\TSM\baclient'+`$z;`$e=`$x+'Common Files\Tivoli\TSM\api64'+`$z;if(Test-Path `$a){`$f=`$a}elseif(Test-Path `$b){`$f=`$b}elseif(Test-Path `$c){`$f=`$c}elseif(Test-Path `$d){`$f=`$d}elseif(Test-Path `$e){`$f=`$e};cd 'C:\Program Files\IBM\gsk8\bin';.\gsk8capicmd_64 -cert -add -label $certlabel -file `$y\$certfilename -db `$f -stashed`""
# This client side script for Linux
$schedfinallnx = "'z=`"/dsmcert.kdb`";y=`"/opt/tivoli/tsm/client/`";a=`$PASSWORDDIR`$z;b=`$DSM_DIR`$z;c=~`"/IBM/StorageProtect/certs`"`$z;d=`$y`"ba/bin64`"`$z;e=`$y`"ba/bin`"`$z;f=`$y`"api/bin64`"`$z;g=`$y`"api/bin`"`$z;if [ -f `$a ];then h=`$a;elif [ -f `$b ];then h=`$b;elif [ -f `$c ];then h=`$c;elif [ -f `$d ];then h=`$d;elif [ -f `$e ];then h=`$e;elif [ -f `$e ];then h=`$f;else h=`$g;fi;/usr/local/ibm/gsk8_64/bin/gsk8capicmd_64 -cert -add -label $certlabel -file $certfilename -db `$h -stashed'"
# This client side script for Aix
$schedfinalaix = "'z=`"/dsmcert.kdb`";y=`"/usr/tivoli/tsm/client/`";a=`$PASSWORDDIR`$z;b=`$DSM_DIR`$z;d=`$y`"ba/bin64`"`$z;e=`$y`"ba/bin`"`$z;f=`$y`"api/bin64`"`$z;g=`$y`"api/bin`"`$z;if [ -f `$a ];then h=`$a;elif [ -f `$b ];then h=`$b;elif [ -f `$d ];then h=`$d;elif [ -f `$e ];then h=`$e;elif [ -f `$e ];then h=`$f;else h=`$g;fi;/usr/opt/ibm/gsk8_64/bin/gsk8capicmd_64 -cert -add -label $certlabel -file $certfilename -db `$h -stashed'"
# This client side script for Mac
$schedfinalmac = "'z=dsmcert.kdb;y=/Library/Application\ Support/tivoli/tsm/client/;a=`${PASSWORDDIR}/`${z};b=`${DSM_DIR}/`${z};e=`${y}ba/bin/`${z};g=`${y}api/bin/`${z};h=`"`";if [ -f `"`$a`" ]; then h=`$a;elif [ -f `"`$b`" ]; then h=`$b;elif [ -f `"`$e`" ]; then h=`$e;else h=`$g;fi;/Library/ibm/gsk8/bin/gsk8capicmd -cert -add -label $certlabel -file $certfilename -db `"`$h`" -stashed'"

$platformstrwin = "WIN"
$platformstrlnx = "LNX"
$platformstraix = "AIX"
$platformstrmac = "MAC"
$platformstrtbd = "TBD"

#Log header
"****************************************" | Out-File -Append $logfile
"Run with action: "+$action+" at: "+(Get-Date -format "yyyy-MM-dd-HH:mm:ss")+"`n" | Out-File -Append $logfile

# Shared variables
# ----------------

if ($action -eq $actiondistribute -or $action -eq $actioncleanup)
{
	# Remove old nodelist if has any
	if (Test-Path -path $nodelistfile)
	{
		Remove-Item -Path $nodelistfile
	}

	# Generate nodelist and domainlist
	$sql = "select distinct nd.domain_name, case when upper(substr(nd.platform_name,1,3))='WIN' or upper(substr(nd.client_os_name,1,3))='WIN' or upper(substr(nd1.platform_name,1,3))='WIN' or upper(substr(nd1.client_os_name,1,3))='WIN' or substr(fs.FILESPACE_NAME,1,1)='\' then 'WIN' when upper(substr(nd.platform_name,1,3)) in ('UNI','AIX') or upper(substr(nd1.client_os_name,1,3))='AIX' or upper(substr(nd1.platform_name,1,3)) in ('UNI','AIX') or upper(substr(nd1.client_os_name,1,3))='AIX' then 'AIX' when upper(substr(nd.platform_name,1,3))='MAC' or upper(substr(nd.client_os_name,1,3))='MAC' or upper(substr(nd1.platform_name,1,3))='MAC' or upper(substr(nd1.client_os_name,1,3))='MAC' then 'MAC' when upper(substr(nd.platform_name,1,3)) in ('LIN','LNX') or upper(substr(nd.client_os_name,1,3)) in ('LNX','SOL','HPX') or upper(substr(nd1.platform_name,1,3)) in ('LIN','LNX') or upper(substr(nd1.client_os_name,1,3)) in ('LNX','SOL','HPX') or substr(fs.FILESPACE_NAME,1,1)='/' then 'LNX' else 'TBD' end as platform, nd.node_name from nodes nd left join filespaces fs on fs.NODE_NAME=nd.NODE_NAME left join nodes nd1 on nd1.proxy_target like '%' || nd.node_name || '%' where nd.nodetype not in ('NAS', 'OBJECTCLIENT') order by nd.domain_name, nd.node_name"
	.\dsmadmc -id="$id" -pass="$pass" -DATAONLY=yes -COMMAdelimited -DISPLaymode=TABle -optfile="$dsmoptfile" -outfile="$nodelistfile" "$sql"

	# Log admadmc result
	Get-Content -Path $nodelistfile | Out-File -Append $logfile

	$nodelist = Get-Content -Path $nodelistfile | sort -Unique

	#$templist = $nodelist | sort -Unique
	$templist = [Object[]]::new($nodelist.count)
	$k = 0
	foreach ($i in $nodelist)
	{
		$s = $i.Split(",")
		$templist[$k] = $s[0]+","+$s[1]
		$k ++
	}
	$domainlist = $templist | sort -Unique
}

# Functions
# ----------------

function distribute_new()
{
	# Remove old macro if has any
	if (Test-Path -path $macrofile)
	{
		Remove-Item -Path $macrofile
	}

	# Generate schedule command strings
	$certline = Get-Content -Path $newcertfile
	$schedcmd = ""
	$schedlist = "","",""
	for ($k=0; $k -le 20; $k++ )
	{
		# Start new schedule at line 0, 7, 14
		if ($k -eq 0 -or $k -eq 7 -or $k -eq 14)
		{
			$schedcmd = $schedbase + "'ECHO_CMD `"" + $certline[$k]
		}
		else
		{
			$schedcmd += "NEW_LINE" + $certline[$k]
		}

		# End current schedule at line 6, 13, 20
		if ($k -eq 6)
		{
			$schedidx = ($k-6)/7
			$schedname = $schednamebase + $schedidx.ToString()
			
			$schedcmd += "`" \> $certfilename'"
			$schedlist[$schedidx] = $schedcmd.replace("SCHED_NAME",$schedname).replace("START_TIME", $schedstartbase+($schedidx*$scheddelay).ToString())
		}
		elseif ($k -eq 13 -or $k -eq 20)
		{
			$schedidx = ($k-6)/7
			$schedname = $schednamebase + $schedidx.ToString()
			
			$schedcmd += "`" \>> $certfilename'"
			$schedlist[$schedidx] = $schedcmd.replace("SCHED_NAME",$schedname).replace("START_TIME", $schedstartbase+($schedidx*$scheddelay).ToString())
		}
	}

	# Define schedule for each domain, platform
	foreach ($i in $domainlist)
	{
		$dom = $i.Split(",")
		foreach ($sched in $schedlist)
		{
			$preDom = $dom[0]
			$schedcmd = $sched.replace("DOMAIN_NAME", $dom[0])
			if ($dom[1] -eq $platformstrwin)
			{
				$schedcmd.replace("ECHO_CMD", "powershell echo").replace("NEW_LINE", "``n").replace("PLATFORM_NAME", $platformstrwin).replace("BEGIN CERTIFICATE", "BEGIN`` CERTIFICATE").replace("END CERTIFICATE", "END`` CERTIFICATE") | Out-File -Encoding ASCII -Append $macrofile
			}
			elseif ($dom[1] -eq $platformstrlnx)
			{
					$schedcmd.replace("ECHO_CMD", "echo -e").replace("NEW_LINE", "\n").replace("PLATFORM_NAME", $platformstrlnx) | Out-File -Encoding ASCII -Append $macrofile
			}
			elseif ($dom[1] -eq $platformstraix)
			{
					$schedcmd.replace("ECHO_CMD", "echo").replace("NEW_LINE", "\n").replace("PLATFORM_NAME", $platformstraix) | Out-File -Encoding ASCII -Append $macrofile
			}
			elseif ($dom[1] -eq $platformstrmac)
			{
					$schedcmd.replace("ECHO_CMD", "echo").replace("NEW_LINE", "\n").replace("PLATFORM_NAME", $platformstrmac) | Out-File -Encoding ASCII -Append $macrofile
			}
			else
			{
				# Platform TBD line, skip it so far
			}
		}

		# Generate the gsk commands
		if ($dom[1] -eq $platformstrwin)
		{
			$schedbase.replace("DOMAIN_NAME", $dom[0]).replace("PLATFORM_NAME", $platformstrwin).replace("SCHED_NAME", $schednamebase+"3").replace("START_TIME", $schedstartbase+(3*$scheddelay).ToString())+$schedfinalwin | Out-File -Encoding ASCII -Append $macrofile
		}
		elseif ($dom[1] -eq $platformstrlnx)
		{
			$schedbase.replace("DOMAIN_NAME", $dom[0]).replace("PLATFORM_NAME", $platformstrlnx).replace("SCHED_NAME", $schednamebase+"3").replace("START_TIME", $schedstartbase+(3*$scheddelay).ToString())+$schedfinallnx | Out-File -Encoding ASCII -Append $macrofile
		}
		elseif ($dom[1] -eq $platformstraix)
		{
			$schedbase.replace("DOMAIN_NAME", $dom[0]).replace("PLATFORM_NAME", $platformstraix).replace("SCHED_NAME", $schednamebase+"3").replace("START_TIME", $schedstartbase+(3*$scheddelay).ToString())+$schedfinalaix | Out-File -Encoding ASCII -Append $macrofile
		}
		elseif ($dom[1] -eq $platformstrmac)
		{
			$schedbase.replace("DOMAIN_NAME", $dom[0]).replace("PLATFORM_NAME", $platformstrmac).replace("SCHED_NAME", $schednamebase+"3").replace("START_TIME", $schedstartbase+(3*$scheddelay).ToString())+$schedfinalmac | Out-File -Encoding ASCII -Append $macrofile
		}
		else
		{
			# Platform TBD line, skip it so far
		}
	}

	# Define assocaition for each node, based on its domain, platform
	$assocmdbase = "def assoc DOMAIN_NAME SCHED_NAME_PLATFORM_NAME NODE_NAME"
	foreach ($i in $nodelist)
	{
		$node = $i.Split(",")
		$assocmd = $assocmdbase.replace("DOMAIN_NAME", $node[0]).replace("NODE_NAME", $node[2])
		if ($node[1] -eq $platformstrwin -or $node[1] -eq $platformstrlnx -or $node[1] -eq $platformstraix -or $node[1] -eq $platformstrmac)
		{
			for ($k=0;$k -le 3; $k++)
			{
				$assocmd.replace("PLATFORM_NAME", $node[1]).replace("SCHED_NAME", $schednamebase+$k) | Out-File -Encoding ASCII -Append $macrofile
			}
		}
		else
		{
			# Platform TBD line, skip it so far
		}
	}

	# Run define macrofile
	$result = (.\dsmadmc -id="$id" -pass="$pass" -optfile="$dsmoptfile" -ITEMCOMMIT macro "$macrofile")
	$result | Select-String "ANS8002I"

	# Log admadmc result
	$result | Out-File -Append $logfile

	$out  = "`n>> Schedules are defined to distribute the certificate to clients.`n"
	$out += ">> A return code other than 0 indicates an error. For more information, take a look at the log file.`n"
	$out += ">> The log file is located at $logfile`n"
	$out += ">> Client nodes will begin receiving a new certificate after $startdelay hours from the start time of the scheduler window.`n"
	$out += ">> To monitor the progress of certificate distribution, it is recommended to run this script in Report mode regularly for a couple of days."

	$out
	$out | Out-File -Append $logfile
}

function distribute_report()
{
	$reportfile = $reportprefix + (Get-Date -format "yyyy-MM-dd-HH_mm_ss")
	$schedpattern = $schednamebase+"3_"

	$sql = "select status, count(*) node_count from (select distinct nd.domain_name, nd.node_name, case when min(ev.status) is not null then min(ev.status) else 'Unknown' end status from nodes nd left join events ev on (ev.domain_name=nd.domain_name and ev.node_name=nd.node_name) where ev.schedule_name is null or upper(ev.schedule_name) like upper(`'$schedpattern%`') group by nd.domain_name,nd.node_name) group by status"
	$result = (.\dsmadmc -id="$id" -pass="$pass" -DISPLaymode=TABle -optfile="$dsmoptfile" -outfile="$tempfile" "$sql")

	Get-Content -Path $tempfile
	Get-Content -Path $tempfile | Out-File $reportfile
	
	$sql = "select distinct nd.domain_name, nd.node_name, case when min(ev.status) is not null then min(ev.status) else 'Unknown' end status from nodes nd left join events ev on (ev.domain_name=nd.domain_name and ev.node_name=nd.node_name) where ev.schedule_name is null or upper(ev.schedule_name) like upper(`'$schedpattern%`') group by nd.domain_name,nd.node_name"
	.\dsmadmc -id="$id" -pass="$pass" -DISPLaymode=TABle -optfile="$dsmoptfile" -outfile="$tempfile" "$sql"

	Get-Content -Path $tempfile | Out-File -Append $reportfile
	
	$out  = ">> The detailed status report is : $reportfile.`n"
	$out += ">> Any status other than 'Completed' indicates a failure.`n"
	$out += ">> Client nodes with status 'Missed' specifies that the scheduled startup window is already passed.`n"
	$out += ">> Client nodes with status 'Future' specifies that the beginning of the startup window is in the future.`n"
	$out += ">> For 'Missed' and 'Future' status there is a chance that they will catch up as these schedules will be re-run on a daily basis.`n"
	$out += ">> Please regularly monitor the progress of certificate distribution status, by executing this script in Report mode for a couple of days.`n"
	$out += ">> Client nodes that are still in failed state, it is recommended to follow the manual steps for adding the certificate.`n"
	$out += ">>     1. Copy the new certificate file over to the client box.`n"
	$out += ">>     2. Execute below command to add the certificate.`n"
	$out += ">>        gsk8capicmd_64 -cert -add -label `"<<New Certificate Label>>`" -file `"<<New Certificate File Path>>`" -db dsmcert.kdb -stashed`n"
	$out += ">>        Tip: The dsmcert.kdb file is located in client installation directory. On Unix, Linux and Mac systems, if client sessions were ever started from a non-root user, copies of the certificate can be located in `$HOME/IBM/StorageProtect/certs/ or in the directory determined by the PASSWORDDIR client option."

	$out
	$out | Out-File -Append $logfile
}

function distribute_cleanup()
{
	# Remove old macro if has any
	if (Test-Path -path $macrofile)
	{
		Remove-Item -Path $macrofile
	}

	$delschebase = "delete sched DOMAIN_NAME SCHED_NAME_PLATFORM_NAME"
	$unixDone = 0
	$preDom = ""
	foreach ($i in $domainlist)
	{
		$dom = $i.Split(",")
		$delcmd = $delschebase.replace("DOMAIN_NAME", $dom[0])
		if ($dom[1] -eq $platformstrwin -or $dom[1] -eq $platformstrlnx -or $dom[1] -eq $platformstraix -or $dom[1] -eq $platformstrmac)
		{
			for ($k=0;$k -le 3; $k++)
			{
				$delcmd.replace("PLATFORM_NAME", $dom[1]).replace("SCHED_NAME", $schednamebase+$k) | Out-File -Encoding ASCII -Append $macrofile
			}
		}
		else
		{
			# Platform TBD line, skip it so far
		}
	}	

	# Run delete macrofile
	$result = (.\dsmadmc -id="$id" -pass="$pass" -optfile="$dsmoptfile" -ITEMCOMMIT macro "$macrofile")
	$result | Select-String "ANS8002I"

	# Log admadmc result
	$result | Out-File -Append $logfile

	$out  = "`n>> Cleanup Completed.`n"
	$out += ">> A return code other than 0 indicates an error. For more information, take a look at the log file.`n"
	$out += ">> The log file is located at $logfile"

	$out
	$out | Out-File -Append $logfile
}

function print_usage()
{
	$out  = "Unknown Action type: $action`n"
	$out += "To start a certification distribute job:`n"
	$out += "  powershell .\cert_distribute.ps1 -id dsmadmc_id -pass dsmadmc_password -action $actiondistribute`n"
	$out += "To monitor status of existing certification distribute job:`n"
	$out += "  powershell .\cert_distribute.ps1 -id dsmadmc_id -pass dsmadmc_password -action $actionreport`n"
	$out += "To cleanup existing certification distribute job:`n"
	$out += "  powershell .\cert_distribute.ps1 -id dsmadmc_id -pass dsmadmc_password -action $actioncleanup`n"

	$out
	$out | Out-File -Append $logfile
}


# Main
# ----------------

if ($action -eq $actiondistribute)
{
	distribute_new
}
elseif ($action -eq $actionreport)
{
	distribute_report
}
elseif ($action -eq $actioncleanup)
{
	distribute_cleanup
}
else
{
	print_usage
}

cd $workingdir
